package org.greenstone.gatherer.gui;

import java.awt.*;
import java.util.regex.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import org.greenstone.gatherer.DebugStream;


public class URLField
    extends JTextField 
    implements DocumentListener {

    static final public Pattern URL_PATTERN = Pattern.compile("(h|ht|htt|http|http:|http:/|http://|http://)(([a-zA-Z]([a-zA-Z]|[0-9]|[\\$\\-_\\&\\+]|[!\\*\"\'\\(\\)]|(%[0-9A-Fa-f][0-9A-Fa-f]))*(\\.[a-zA-Z]([a-zA-Z]|[0-9]|[\\$\\-_\\&\\+]|[!\\*\"\'\\(\\)]|(%[0-9A-Fa-f][0-9A-Fa-f]))*)*)|(([0-9])+\\.([0-9])+\\.([0-9])+\\.([0-9])+))(:([0-9])+)?(/([a-zA-Z]|[0-9]|[\\$\\-_\\.\\&\\+]|[!\\*\"\'\\(\\),]|(%[0-9A-Fa-f][0-9A-Fa-f]))*)*");

    // Crap RegEx that matches everything dammit
    //static final public Pattern URL_PATTERN = Pattern.compile("((.*?)://)?(([^:]*):([^@]*)@)?(([^/:]*)(:[^/]*))?([^\\?#]*/?)?(\\?([^?#]*))?(#(.*))?");

    // Crap RegEx that demands URL starts with www
    //static final public Pattern URL_PATTERN = Pattern.compile("(h|ht|htt|http|http:|http:/|http://|http://)?[\\w]+(\\.[\\w]+)([\\w\\-\\.,@?^=%&:/~\\+#]*[\\w\\-\\@?^=%&/~\\+#])?");

    private Color background;
    private Color foreground;
    private Color invalid_background;
    private Color invalid_foreground;

    public URLField(Color foreground, Color background, Color invalid_foreground, Color invalid_background) {
	super();
	this.background = background;
	this.foreground = foreground;
	this.invalid_background = invalid_background;
	this.invalid_foreground = invalid_foreground;
	setBackground(background);
	setForeground(foreground);
	//getDocument().addDocumentListener(this);
    }

    public URLField(String email, Color foreground, Color background, Color invalid_foreground, Color invalid_background) {
	super(email);
	this.background = background;
	this.foreground = foreground;
	this.invalid_background = invalid_background;
	this.invalid_foreground = invalid_foreground;
	setBackground(background);
	//getDocument().addDocumentListener(this);
    }

    /** Gives notification that an attribute or set of attributes changed.
     * @param e
     */
    public void changedUpdate(DocumentEvent e) {
	validateURL();
    }
          
    /** Gives notification that there was an insert into the document.
     * @param e
     */
    public void insertUpdate(DocumentEvent e) {
	validateURL();
    }

    public boolean isEmpty() {
	return (this.getText().length() == 0);
    }
          
    /** Gives notification that a portion of the document has been removed. 
     * @param e
     */
    public void removeUpdate(DocumentEvent e) {
	validateURL();
    }

    public boolean validateURL() {
	boolean result;
	String text = getText();
	Matcher m = URL_PATTERN.matcher(text);
	if(text.length() == 0 || m.matches()) {
	    setBackground(background);
	    setForeground(foreground);
	    result = true;
	}
	else {
	    setBackground(invalid_background);
	    setForeground(invalid_foreground);
	    result = false;
	}
	m = null;
	DebugStream.println("Validating: " + text + " -> " + (result ? "GOOD" : "BAD"));
	text = null;
	return result;
    }

    static public void main(String[] args) {
	JFrame frame = new JFrame("URLField test");
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	frame.setSize(300,60);
	JPanel panel = (JPanel) frame.getContentPane();
	panel.setLayout(new BorderLayout());
	panel.add(new URLField(Color.black, Color.white, Color.white, Color.red), BorderLayout.CENTER);
	frame.setVisible(true);
    }
}
